@@ -4,41 +4,27 @@ from nose.util import tolist
 import os
 import ConfigParser
 import logging
+import codecs
 
 log = logging.getLogger(__name__)
 
 warning = "Cannot access the test config because the plugin has not \
 been activated.  Did you specify --tc or any other command line option?"
 
-class _uninitialized_config(object):
-    """a dummy config until the plugin creates a real one"""
-    def _warn(self):
-        import warnings
-        warnings.warn(warning, RuntimeWarning)
-
-    def __getitem__(self, item):
-        self._warn()
-        return None
-
-    def get(self, item, default=None):
-        self._warn()
-        return None
+config = {}
 
 
-config = _uninitialized_config()
-
-
-def load_yaml(yaml_file):
+def load_yaml(yaml_file, encoding):
     """ Load the passed in yaml configuration file """
     try:
         import yaml
     except (ImportError):
         raise Exception('unable to import YAML package. Can not continue.')
     global config
-    config = yaml.load(open(yaml_file).read())
+    config = yaml.load(codecs.open(yaml_file, 'r', encoding).read())
 
 
-def load_ini(ini_file):
+def load_ini(ini_file, encoding):
     """ Parse and collapse a ConfigParser-Style ini file into a nested,
     eval'ing the individual values, as they are assumed to be valid
     python statement formatted """
@@ -46,7 +32,9 @@ def load_ini(ini_file):
     import ConfigParser
     global config
     tmpconfig = ConfigParser.ConfigParser()
-    tmpconfig.read(ini_file)
+    with codecs.open(ini_file, 'r', encoding) as f:
+        tmpconfig.readfp(f)
+
     config = {}
     for section in tmpconfig.sections():
         config[section] = {}
@@ -54,11 +42,20 @@ def load_ini(ini_file):
             config[section][option] = tmpconfig.get(section, option)
 
 
-def load_python(py_file):
+def load_python(py_file, encoding):
     """ This will exec the defined python file into the config variable - 
     the implicit assumption is that the python is safe, well formed and will
     not do anything bad. This is also dangerous. """
-    exec(open(py_file, 'r'))
+    exec(codecs.open(py_file, 'r', encoding).read())
+
+
+def load_json(json_file, encoding):
+    """ This will use the json module to to read in the config json file.
+    """
+    import json
+    global config
+    with codecs.open(json_file, 'r', encoding=encoding) as handle:
+        config = json.load(handle)
 
 
 class TestConfig(Plugin):
@@ -69,8 +66,9 @@ class TestConfig(Plugin):
 
     env_opt = "NOSE_TEST_CONFIG_FILE"
     format = "ini"
+    encoding = 'utf-8'
     valid_loaders = { 'yaml' : load_yaml, 'ini' : load_ini,
-                      'python' : load_python }
+                      'python' : load_python, 'json': load_json }
 
     def options(self, parser, env=os.environ):
         """ Define the command line options for the plugin. """
@@ -80,6 +78,12 @@ class TestConfig(Plugin):
             dest="testconfig",
             help="Configuration file to parse and pass to tests"
                  " [NOSE_TEST_CONFIG_FILE]")
+        parser.add_option(
+            "--tc-file-encoding", action="store",
+            default=env.get('NOSE_TEST_CONFIG_FILE_ENCODING') or self.encoding, 
+            dest="testconfigencoding",
+            help="Test config file encoding, default is utf-8"
+                 " [NOSE_TEST_CONFIG_FILE_ENCODING]")
         parser.add_option(
             "--tc-format", action="store",
             default=env.get('NOSE_TEST_CONFIG_FILE_FORMAT') or self.format, 
@@ -87,12 +91,12 @@ class TestConfig(Plugin):
             help="Test config file format, default is configparser ini format"
                  " [NOSE_TEST_CONFIG_FILE_FORMAT]")
         parser.add_option(
-            "--tc", action="append", 
+            "--tc", action="append",
             dest="overrides",
             default = [],
             help="Option:Value specific overrides.")
         parser.add_option(
-            "--tc-exact", action="store_true", 
+            "--tc-exact", action="store_true",
             dest="exact",
             default = False,
             help="Optional: Do not explode periods in override keys to "
@@ -102,7 +106,7 @@ class TestConfig(Plugin):
     def configure(self, options, noseconfig):
         """ Call the super and then validate and call the relevant parser for
         the configuration file passed in """
-        if not options.testconfig:
+        if not options.testconfig and not options.overrides:
             return
         Plugin.configure(self, options, noseconfig)
 
@@ -116,20 +120,27 @@ class TestConfig(Plugin):
                                                                 % self.format)
 
         # Load the configuration file:
-        self.valid_loaders[self.format](options.testconfig)
+        if options.testconfig:
+            self.valid_loaders[self.format](options.testconfig,
+                                            options.testconfigencoding)
 
-        if options.overrides:
-            self.overrides = []
-            overrides = tolist(options.overrides)
-            for override in overrides:
-                keys, val = override.split(":")
-                if options.exact:
-                    config[keys] = val
-                else:                    
-                    ns = ''.join(['["%s"]' % i for i in keys.split(".") ])
-                    # BUG: Breaks if the config value you're overriding is not
-                    # defined in the configuration file already. TBD
-                    exec('config%s = "%s"' % (ns, val))
+        overrides = tolist(options.overrides) or []
+        for override in overrides:
+            keys, val = override.split(":", 1)
+            if options.exact:
+                config[keys] = val
+            else:
+                # Create all *parent* keys that may not exist in the config
+                section = config
+                keys = keys.split('.')
+                for key in keys[:-1]:
+                    if key not in section:
+                        section[key] = {}
+                    section = section[key]
+
+                # Finally assign the value to the last key
+                key = keys[-1]
+                section[key] = val
 
 
 # Use an environment hack to allow people to set a config file to auto-load
@@ -142,4 +153,7 @@ if 'NOSE_TESTCONFIG_AUTOLOAD_INI' in os.environ:
     load_ini(os.environ['NOSE_TESTCONFIG_AUTOLOAD_INI'])
 
 if 'NOSE_TESTCONFIG_AUTOLOAD_PYTHON' in os.environ:
-    load_python(os.environ['NOSE_TESTCONFIG_AUTOLOAD_PYTHON'])
\ No newline at end of file
+    load_python(os.environ['NOSE_TESTCONFIG_AUTOLOAD_PYTHON'])
+
+if 'NOSE_TESTCONFIG_AUTOLOAD_JSON' in os.environ:
+    load_json(os.environ['NOSE_TESTCONFIG_AUTOLOAD_JSON'])
